from sage.modular.jacobi.classical import *

prec = 7

def reduce_qf(qf):
  bqf = BinaryQF(qf)

  if bqf.discriminant() == 0:
    n = gcd([bqf[0],bqf[1],bqf[2]])
    return (n,0,0)
  else:
    bqf = bqf.reduced_form()
    return (bqf[2], abs(bqf[1]), bqf[0])

def complete_expansion(d, k, m_min, prec):
  ## we assume that the 0th Fourier Jacobi coefficient is given
  for m in range(m_min,prec):
    jforms = classical_jacobi_forms(k,m,prec)
    
    ## create matrix from Fourier expansion
    nrs = [(n,r) for n in range(prec) for r in range(m+1)
           if 4*n*m - r**2 >= 0]
    nrs_part = [(n,r) for (n,r) in nrs
                      if n < m or reduce_qf((n,r,m)) in d]

    jmat = matrix(QQ, [[phi.get((n,r),0) for (n,r) in nrs]
                       for phi in jforms])
    jmat_part = jmat.matrix_from_columns([
                  ix for (ix,nr) in enumerate(nrs)
                     if nr in nrs_part])
    fe_part = vector([d[reduce_qf((n,r,m))] for (n,r) in nrs
                                            if (n,r) in nrs_part])
    fe_coord = jmat_part.solve_left(fe_part)
    fe = fe_coord * jmat
    
    for (ix,(n,r)) in enumerate(nrs):
      if (n,r,m) in d:
        assert d[(n,r,m)] == fe[ix]
      else:
        d[(n,r,m)] = fe[ix]

  return d

e4_mf = EisensteinForms(1,4).gen(0).qexp(prec)
e4_mf /= e4_mf[0]
e4_pre = dict(((n,0,0), e4_mf[n]) for n in range(prec)) 
e4_fe = complete_expansion(e4_pre, 4, 1, prec)
save(e4_fe, "e4_fe.sobj")

e6_mf = EisensteinForms(1,6).gen(0).qexp(prec)
e6_mf /= e6_mf[0]
e6_pre = dict(((n,0,0), e6_mf[n]) for n in range(prec)) 
e6_fe = complete_expansion(e6_pre, 6, 1, prec)
save(e6_fe, "e6_fe.sobj")

phi2_10_pre = dict([((n,0,0), 0) for n in range(prec)] +
                 [((1,0,1), -2), ((1,1,1), 1)])
phi2_10_fe = complete_expansion(phi2_10_pre, 10, 1, prec)
save(phi2_10_fe, "phi2_10_fe.sobj")

e12_mf = EisensteinForms(1,12).gen(0).qexp(prec)
e12_mf *= 53678953 / e12_mf[0]
e12_pre = dict([((n,0,0), e12_mf[n]) for n in range(prec)] +
               [((1,0,1), 456798756960), ((1,1,1), 22266840960)])
e12_fe = complete_expansion(e12_pre, 12, 1, prec)
save(e12_fe, "e12_fe.sobj")

ed_mf = CuspForms(1,12).gen(0).qexp(prec)
ed_mf *= 12096 / ed_mf[1]
ed_pre = dict([((n,0,0), ed_mf[n]) for n in range(prec)] +
              [((1,0,1), 2146176), ((1,1,1), 158976)])
ed_fe = complete_expansion(ed_pre, 12, 1, prec)
save(ed_fe, "ed_fe.sobj")

phi2_12_pre = dict([((n,0,0), 0) for n in range(prec)] +
                 [((1,0,1), 10), ((1,1,1), 1)])
phi2_12_fe = complete_expansion(phi2_12_pre, 12, 1, prec)
save(phi2_12_fe, "phi2_12_fe.sobj")
