R.<u1,u2,v1,v2> = QQ[]

## quadratic forms of diagonal up to prec
prec = 5
## we have to take quite many indices (n,r,m) into account to
## guarantee correctness of the final result
qfs = [ (n,r,m) for n in range(-1,prec+2)
                for r in range(-4*(prec+1)**2,4*(prec+1)**2+1)
                for m in range(-1,prec+2) ]

## we use a combinatorial algebra to multiply Siegel modular forms
class FEAlgebra(CombinatorialFreeModule):
  def __init__(self):
     super(FEAlgebra,self).__init__(R,qfs,category=AlgebrasWithBasis(R))

  def product_on_basis(self,i,j):
    ij = tuple(map(operator.add,i,j))
    if ij[0] >= prec or ij[2] >= prec:
      return self.zero()
    else:
      return self(ij)

  def one(self):
    return self((0,0,0))

FE = FEAlgebra()

## it is important to note that fRS_phi2_10 and phi2_10_inv have both been stored
## with all Fourier coefficients, not only the reduced ones.
fRS_phi2_10_fe = \
  dict( ((n,r,m),c)
        for ((n,r,m),c) in load("fRS_phi2_10_fe.sobj").items()
        if n < prec+2 and m < prec+2 )
phi2_10_inv_fe = \
  dict( ((n,r,m),c)
        for ((n,r,m),c) in load("phi2_10_inv_fe.sobj").items()
        if n < prec and m < prec and abs(r) < 4*(prec+1)**2 - 4*(prec+1) )


fRS_phi2_10 = sum( FE(e)*c for (e,c) in fRS_phi2_10_fe.items() )
phi2_10_inv = sum( FE(e)*c for (e,c) in phi2_10_inv_fe.items() )

fRS = phi2_10_inv * fRS_phi2_10

fRS_dict = dict(((n,r,m),c) for ((n,r,m),c) in list(fRS)
                            if n < prec and m < prec and abs(r) < 2*prec )
save(fRS_dict, "fRS_fe.sobj")


## This is a simple cross check, to verify the result. Note the final object
## must_be_zero has precision prec-1, because fRS has support on (n,r,m) with
## n=-1 or m=-1

def reduce_qf(qf):
  bqf = BinaryQF(qf)

  if qf[0] < 0 or qf[2] < 0 or bqf.discriminant() > 0:
    return None

  if bqf.discriminant() == 0:
    n = gcd([bqf[0],bqf[1],bqf[2]])
    return (n,0,0)
  else:
    bqf = bqf.reduced_form()
    return (bqf[2], abs(bqf[1]), bqf[0])

def from_dict(d):
  res = FE.zero()
  for qf in qfs:
    qfr = reduce_qf(qf)
    if qfr is not None: res += d[qfr] * FE(qf)
  return res

phi2_10_fe = dict( ((n,r,m),c)
                 for ((n,r,m),c) in load("phi2_10_fe.sobj").items()
                 if n < prec+2 and m < prec+2 )
phi2_10 = from_dict(phi2_10_fe)

must_be_zero = phi2_10 * fRS - fRS_phi2_10

