from sage.modular.jacobi.classical import *

prec = 6
tot_prec = 2*prec + 1  # the precision for multivariate power series rings
phi0_prec = (prec+1)**2 + 1

## compute the input to the Borcherds lift
phi12 = classical_jacobi_forms(12,1,phi0_prec+1)[1]
dinv = 1/13 * CuspForms(1,12).gen(0).qexp(phi0_prec+2).shift(-1)**-1
R = dinv.parent(); q = R.gen(0)

phi0_0 = dinv * sum(q**n * phi12.get((n+1,0),0) for n in range(phi0_prec))
phi0_1 = dinv * sum(q**n * phi12.get((n+1,1),0) for n in range(phi0_prec))

phi0 = {}
for n in range(phi0_prec):
  phi0[(n,0)] = phi0_0[n]
  phi0[(n,1)] = phi0_1[n]


## indices (n,r,m) in the Weyl chamber
nrms = [ (n,r,m)
         for n in range(1,prec+2)
         for m in range(0,prec+2)
         for r in range(-isqrt(4*n*m+1), isqrt(4*n*m+1)+1) ]
nrms += [ (0,r,m)
          for m in range(1,prec+2)
          for r in range(-isqrt(4*0*m+1), isqrt(4*0*m+1)+1) ]
nrms += [ (0,r,0)
          for r in range(-isqrt(4*0*0+1),0) ]

def reduce_nr(n,r):
  r_n = r % 2
  n_n = (4*n - r**2 + r_n**2) // 4
  return (n_n,r_n)

P.<zeta> = LaurentSeriesRing(QQ, default_prec = 4*prec**2+1)
R.<q1,q2> = PowerSeriesRing(P, default_prec = tot_prec)

phi2_10_inv = 1
for (n,r,m) in nrms:
  if n != 0 or m != 0:
    phi2_10_inv *= (1 - q1**n * zeta**r * q2**m + O(q1,q2)**(tot_prec+1)) \
                **(-phi0[reduce_nr(n*m,r)])
  else:
    phi2_10_inv *= (1 - zeta**r)**(-phi0[reduce_nr(n*m,r)])

## only here we add the Weyl vector, which is (-1,-1,-1)
phi2_10_inv_dict_outer = dict((tuple(e),c) for (e,c) in phi2_10_inv.dict().items())
phi2_10_inv_dict = dict( ((n-1,r-1,m-1), c_outer[r])
                      for ((n,m),c_outer) in phi2_10_inv_dict_outer.items()
                      for r in c_outer.exponents()
                      if n-1 < prec and m-1 < prec and abs(r-1) <= 4*prec)

save(phi2_10_inv_dict, "phi2_10_inv_fe.sobj")

